<?php
namespace Ecommerce\Services;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Exception\CustomException;
use Doctrine\ORM\EntityManager;
use Ecommerce\Util\LoginUtilities;
use \Teknisa\Libs\Util\Utilities as LibUtilities;

class LoginServiceEcommerce
{
    const BASE_SUBJECT_EMAIL = 'Recuperação de Acesso';
    const USES_ACTIVE_DIRECTORY_POSITIVE = 'Y';
    
    protected $entityManager;
    protected $criptografia;
    protected $smtpUtil;
    protected $utilities;
    
    public $loginUrl;
    public $loginClientSecret;
    
    public $libClientId;
    public $libClientSecret;

    public function __construct(EntityManager $entityManager, Criptografia $criptografia, $environment, $smtpUtil, LibUtilities $utilities)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->criptografia = $criptografia;
        $this->environment = $environment;
        $this->smtpUtil = $smtpUtil;
        $this->utilities = $utilities;
        
        $config = $this->utilities->getLoginConfig();
        $this->loginUrl = $config['loginUrl'];
        $this->loginClientSecret = $config['loginClientSecret'];
        
        $integration = $this->utilities->getLoginIntegration();
        $this->libClientId = $integration['clientId'];
        $this->libClientSecret = $integration['clientSecret'];
    }
    
    public function beginTransaction(){
        $this->entityManager->beginTransaction();
    }

    public function commitTransaction(){
        $this->entityManager->commit();
    }

    public function rollbackTransaction(){
        $this->entityManager->rollback();
    }

    private function retornaEmpresa($cdforneced)
    {

        return $this->entityManager->getRepository('\Geral\Entities\Fornecedor')
            ->findOneBy(array('cdforneced' => $cdforneced));

    }

    private function iniciaAmbiente($dadosSessao)
    {

        $this->environment->setCdforneced($dadosSessao['CDFORNECED']);
        $this->environment->setDslogineco($dadosSessao['DSLOGINECO']);
        $this->environment->setIdacessocot($dadosSessao['IDACESSOCOT']);
        $this->environment->setIdacessoaf($dadosSessao['IDACESSOAF']);
        $this->environment->setIdacessocr($dadosSessao['IDACESSOCR']);
        $this->environment->setIdadministrador($dadosSessao['IDADMINISTRADOR']);

    }

    private function criaDadosDaSessao($fornecedor, $empresa)
    {

        return array(
            "CDFORNECED" => $fornecedor->getCdforneced(),
            "DSLOGINECO" => $fornecedor->getDslogineco(),
            "IDACESSOCOT" => $fornecedor->getIdacessocot(),
            "IDACESSOAF" => $fornecedor->getIdacessoaf(),
            "IDACESSOCR" => $fornecedor->getIdacessocr(),
            "IDADMINISTRADOR" => $fornecedor->getIdadministrador(),
            "NMFANTFORN" => $empresa->getNmfantforn(),
            "NMRAZSOCFORN" => $empresa->getNmrazsocforn(),
            "NRINSJURFORN" => $empresa->getNrinsjurforn(),
            "IDTPIJURFORN" => $empresa->getIdtpijurforn()
        );

    }

    public function validaSenha($cdsenhaweb, $senha)
    {

        if ($cdsenhaweb !== $this->criptografia->retornaSenhaWeb($senha)) {
            throw new \Exception("Senha inválida.");
        }

    }

    private function retornaFornecedorPorEmail($email)
    {

        $fornecedor = $this->entityManager->getRepository('\Portalecommerce\Entities\Dadosfornecommerce')
            ->findBy(array('dslogineco' => $email));

        if (!$fornecedor) {
            throw new \Exception("Login não está cadastrado.");
        }

        return $fornecedor;

    }

    private function retornaFornecedorEcommerce($cdfornecedor, $email)
    {

        $fornecedor = $this->entityManager->getRepository('\Portalecommerce\Entities\Dadosfornecommerce')
            ->findOneBy(array('cdforneced' => $cdfornecedor, 'dslogineco' => $email));

        if (!$fornecedor) {
            throw new \Exception("Login não está cadastrado.");
        }

        return $fornecedor;

    }

    private function retornaFornecedorFilho($cdfornecedor)
    {

        $filhos = $this->entityManager->getRepository('\Geral\Entities\Fornecedor')
            ->findBy(array('cdfornerelacaf' => $cdfornecedor));

        $empresa = $this->entityManager->getRepository('\Geral\Entities\Fornecedor')
            ->findOneBy(array('cdforneced' => $cdfornecedor));

        $nrinsjurforn = $empresa->getNrinsjurforn();
        $nmrazsocforn = $empresa->getNmrazsocforn();
        $idtpijurforn = $empresa->getIdtpijurforn();

        $filhosArray = array();

        $filhosArray[] = array(
            'CDFORNECEDOR' => $this->fornecedor->getCdforneced(),
            'NRINSJURFORN' => $nrinsjurforn,
            'NMRAZSOCFORN' => $nmrazsocforn,
            'IDTPIJURFORN' => $idtpijurforn,
            'INSJURRAZSOC' => $nrinsjurforn . ' - ' . $nmrazsocforn
        );

        foreach ($filhos as $filho) {

            $nrinsjurfornFilho = $filho->getNrinsjurforn();
            $nmrazsocfornFilho = $filho->getNmrazsocforn();
            $idtpijurfornFilho = $empresa->getIdtpijurforn();

            $filhosArray[] = array(
                'CDFORNECEDOR' => $filho->getCdforneced(),
                'NRINSJURFORN' => $nrinsjurfornFilho,
                'NMRAZSOCFORN' => $nmrazsocfornFilho,
                'IDTPIJURFORN' => $idtpijurfornFilho,
                'INSJURRAZSOC' => $nrinsjurfornFilho . ' - ' . $nmrazsocfornFilho
            );

        }

        return $filhosArray;

    }

    public function autenticar($cdfornecedor, $email, $senha)
    {

        $this->fornecedor = $this->retornaFornecedorEcommerce($cdfornecedor, $email);

        $this->validaSenha($this->fornecedor->getCdsenhaweb(), $senha);

        $empresa = $this->retornaEmpresa($this->fornecedor->getCdforneced());

        $dadosDaSessao = $this->criaDadosDaSessao($this->fornecedor, $empresa);

        $fornecedorFilho = $this->retornaFornecedorFilho($this->fornecedor->getCdforneced());

        $parametros = $this->retornaParamGeral($empresa->getNrorg());

        $this->iniciaAmbiente($dadosDaSessao);

        return array($dadosDaSessao, $fornecedorFilho, $parametros);

    }

    private function criaEmail($fornecedor)
    {

        $cdrandomacesso = $fornecedor->getCdrandomacesso();
        $email = $fornecedor->getDslogineco();

        return array(
            "assunto" => "Portal E-Commerce - Redefinição de senha.",
            "email" => $email,
            "nome" => $email,
            "mensagem" => "Olá\n\nRecebemos uma solicitação de redefinição de senha para sua conta no PortalECommerce. Para redefinir sua senha, informe o código abaixo:\n\n$cdrandomacesso\n\nSe você não solicitou uma redefinição de senha, desconsidere este email.\n\nA equipe do Portal E-Commerce."
        );

    }

    private function atualizarCodigoVerificacao($fornecedores, $codigoVerificacao)
    {
        foreach ($fornecedores as $fornecedor) {
            $fornecedor->setCdrandomacesso($codigoVerificacao);
            $this->entityManager->persist($fornecedor);
        }
        $this->entityManager->flush();
    }

    private function retornaCodigoVerificacao($length = 10)
    {

        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;

    }

    public function checaEmail($email)
    {

        $fornecedor = $this->retornaFornecedorPorEmail($email);

        $codigoVerificacao = $this->retornaCodigoVerificacao(6);

        $this->atualizarCodigoVerificacao($fornecedor, $codigoVerificacao);

        $email = $this->criaEmail($fornecedor[0]);
        
        $this->smtpUtil->setFromName('Portal E-Commerce');
        $this->smtpUtil->setSubject($email['assunto']);
        $this->smtpUtil->setNmAddress($email['nome']);
        $this->smtpUtil->sendMail($email['mensagem'], $email['email']);
        
        return $fornecedor;

    }

    private function verificaCodigo($codigoSalvo, $codigoVerificacao)
    {

        if ($codigoSalvo !== $codigoVerificacao) {
            throw new \Exception("Código inválido.");
        }

    }

    public function addNovaSenha($email, $codigoVerificacao, $novaSenha)
    {

        $fornecedores = $this->retornaFornecedorPorEmail($email);

        $this->verificaCodigo($fornecedores[0]->getCdrandomacesso(), $codigoVerificacao);

        $this->salvaSenha($fornecedores, $novaSenha);

        return true;

    }

    public function alteraSenha($email, $novaSenha)
    {

        $fornecedores = $this->retornaFornecedorPorEmail($email);

        $this->salvaSenha($fornecedores, $novaSenha);

    }

    private function salvaSenha($fornecedores, $novaSenha)
    {
        foreach ($fornecedores as $fornecedor) {
            $fornecedor->setCdsenhaweb($this->criptografia->retornaSenhaWeb($novaSenha));
            $fornecedor->setCdsenha($this->utilities->getDelphiPasswordEncrypted($novaSenha));
            $this->entityManager->persist($fornecedor);
        }
        $this->entityManager->flush();
    }

    public function buscaUsuarioLogado($email)
    {

        $fornecedor = $this->retornaFornecedorPorEmail($email);

        return $fornecedor;

    }

    private function retornaParamGeral($nrorg)
    {


        $param = $this->entityManager->getRepository('Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $nrorg));


        $idsisdos = $param->getIdsisdos();
        $nrorg = $param->getNrorg();
        $cdpictprod = $param->getCdpictprod();

        $parametrosArray = array(
            'IDSISDOS' => $idsisdos,
            'NRORG' => $nrorg,
            'CDPICTPROD' => $cdpictprod
        );

        return $parametrosArray;

    }
    
    /*******************************/
    /***** NEW LOGIN FUNCTIONS *****/
    /*******************************/
    
    public function getUserData($row)
    {
        try {
            $sql = "SELECT CDOPERFORN AS CDOPERADOR, CDFORNECED, DSLOGINECO, CDSENHAWEB, CDSENHA, IDACESSOCOT, IDACESSOAF, 
                          IDACESSOCR, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, 
                          NRORGULTATU, CDOPERULTATU, DSEMAILECO, IDADMINISTRADOR, NMUSUARIOECO AS NMOPERADOR, IDACESSOUPLOAD
                      FROM DADOSFORNECOMMERCE
                     WHERE DSLOGINECO = :P_DSLOGINECO
                      AND CDSENHAWEB = :P_CDSENHAWEB";
                    
            $params = array(
                "P_DSLOGINECO" => $row["USER_EMAIL"],
                "P_CDSENHAWEB" => $this->criptografia->retornaSenhaWeb($row["PASSWORD"])
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);

            return $std->fetchAll();
        } catch(\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getOrganizationByEmail($row){
        $sql = "SELECT NRORG FROM DADOSFORNECOMMERCE WHERE DSLOGINECO = :P_DSLOGINECO";
        
        $params = array(
            "P_DSLOGINECO" => $row['EMAIL']
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetch();
    }
    
    public function checkUserExist($email)
    {
        try {
            $sql = "SELECT 
                        CDFORNECED,
                        DSLOGINECO,
                        NMUSUARIOECO
                    FROM 
                        DADOSFORNECOMMERCE
                    WHERE
                        DSLOGINECO = :P_DSLOGINECO";
                    
            $params = array(
                "P_DSLOGINECO" => $email
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);

            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function inactivateOtherRequests($row, $token)
    {
        try {
            $sql = "UPDATE 
                        CONTROLSENHAECOMMERCE
                    SET
                        IDATIVO = 'N'
                    WHERE
                        DSEMAILECOMMERCE = :P_DSEMAILECOMMERCE
                        AND DSTOKEN != :P_DSTOKEN";
                    
            $params = array(
                "P_DSEMAILECOMMERCE" => $row["EMAIL"],
                "P_DSTOKEN" => $token
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function insertLogRequestPassword($row, $token)
    {
        try {
            $sql = "INSERT INTO CONTROLSENHAECOMMERCE
                    (
                        DSEMAILECOMMERCE,
                        DSTOKEN,
                        IDATIVO,
                        DTREQUISICAO
                    ) 
                    VALUES
                    (
                        :P_DSEMAILECOMMERCE,
                        :P_DSTOKEN,
                        'S',
                        SYSDATE
                    )";
                    
            $params = array(
                "P_DSEMAILECOMMERCE" => $row["EMAIL"],
                "P_DSTOKEN" => $token
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function getRequestStatus($row)
    {
        try {
            $sql = "SELECT 
                        IDATIVO,
                        DTREDEFINICAO
                    FROM 
                        CONTROLSENHAECOMMERCE
                    WHERE
                        DSEMAILECOMMERCE = :P_DSEMAILECOMMERCE
                        AND DSTOKEN = :P_DSTOKEN";
                    
            $params = array(
                "P_DSEMAILECOMMERCE" => $row["EMAIL"],
                "P_DSTOKEN" => $row["TOKEN"]
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);

            return $std->fetchAll();
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function updateLogRequestPassword($row)
    {
        try {
            $sql = "UPDATE 
                        CONTROLSENHAECOMMERCE
                    SET
                        IDATIVO = 'N',
                        DTREDEFINICAO = SYSDATE
                    WHERE
                        DSEMAILECOMMERCE = :P_DSEMAILECOMMERCE
                        AND DSTOKEN = :P_DSTOKEN";
                    
            $params = array(
                "P_DSEMAILECOMMERCE" => $row["EMAIL"],
                "P_DSTOKEN" => $row["TOKEN"]
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function redefinePassword($row)
    {
        try {
            $params = array(
                $this->criptografia->retornaSenhaWeb($row["PASSWORD"]),
                $this->utilities->getDelphiPasswordEncrypted($row["PASSWORD"]),
                $row["EMAIL"]
            );
            $this->updatePasswordDadosFornEcommerce(...$params);
            $this->updatePasswordOperador(...$params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    
    private function updatePasswordDadosFornEcommerce($passwordWeb, $passwordDelphi, $email){
        $sql = "UPDATE DADOSFORNECOMMERCE SET
                    CDSENHAWEB = :P_CDSENHAWEB,
                    CDSENHA    = :P_CDSENHA
                WHERE
                    DSLOGINECO = :P_DSLOGINECO";
                
        $params = array(
            "P_CDSENHAWEB" => $passwordWeb,
            "P_CDSENHA"    => $passwordDelphi,
            "P_DSLOGINECO" => $email
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    private function updatePasswordOperador($passwordWeb, $passwordDelphi, $email){
        $sql = "UPDATE OPERADOR SET
                    CDSENHOPER     = :P_CDSENHOPER,
                    CDSENHAOPERWEB = :P_CDSENHAOPERWEB
                WHERE
                    DSEMAILOPER    = :P_DSEMAILOPER";
        $params = array(
            "P_CDSENHOPER"     => $passwordDelphi,
            "P_CDSENHAOPERWEB" => $passwordWeb,
            "P_DSEMAILOPER"    => $email
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getLibConnectionCustom()
    {
        return InstanceManager::getInstance()->getService('connectionCustom');
    }
    
    public static function getLibMenuService()
    {
        return InstanceManager::getInstance()->getService('libMenuService');
    }
    
    public function getLibServiceApi()
    {
        return InstanceManager::getInstance()->getService('apiService');
    }
    
    public function getAuthImpl()
    {
        return InstanceManager::getInstance()->getService('oAuthImpl');
    }
    
    public function getSession()
    {
        return InstanceManager::getInstance()->getService('session');
    }
    
    public static function getOAuthCustom()
    {
        return InstanceManager::getInstance()->getService('oAuthImpl');
    }
    
    public static function getLibEnvironment()
    {
        return InstanceManager::getInstance()->getService('libEnvironment');
    }
    
    /**
     * @param $email
     * @param $style
     * @throws CustomException
     */
    public function requestNewPassword($email, $style) 
    {
        try {
            $userData = $this->getLibServiceApi()->getUserData($email, null);
            
            if (empty($userData) || empty($userData['USER'])) {
                throw new \Exception("Usuário não encontrado.");
                //Exception::userNotFound();
            }
            
            $user = $userData['USER'];
            $email = $user['EMAIL'];
            
            $passwordParams = $userData['PASSWORD_PARAMETERS'];
            $this->validateExistsAdmUser($passwordParams);
            
            $password = $this->utilities->getRandomPassword();
            $this->getLibServiceApi()->updatePassword($email, $password, 'R');
            
            $this->sendEmail($user['ID'], $email, $password, $style);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     * @param $passwordParams
     * @throws \Exception
     */
    private function validateExistsAdmUser($passwordParams) 
    {
        if (isset($passwordParams['USES_ACTIVE_DIRECTORY']) &&
                  $passwordParams['USES_ACTIVE_DIRECTORY'] == self::USES_ACTIVE_DIRECTORY_POSITIVE &&
                  !$passwordParams['ADM_USER_ACTIVE_DIRECTORY']) {
            throw new CustomException('NOT_ADM_USER_REQUEST_NEW_PASSWORD');
        }
    }
    
    /**
     * @param $cdOperator
     * @param $email
     * @param $password
     * @param $style
     */
    private function sendEmail($cdOperator, $email, $password, $style)
    {
        $body = utf8_encode($this->getReplacedTemplate($cdOperator, $email, $password, $style));
        
        $smtp = InstanceManager::getInstance()->getService('smtpUtil');
        $smtp->setSubject(self::BASE_SUBJECT_EMAIL);
        $smtp->setIsHTML(true);
        $smtp->sendMail($body, $email);
    }
    
    /**
     * @param $cdOperator
     * @param $email
     * @param $password
     * @param $style
     * @return mixed|string
     */
    private function getReplacedTemplate($cdOperator, $email, $password, $style) 
    {
        $replacedContents = utf8_decode(file_get_contents($this->getResetPasswordTemplatePath($style)));
        $replacedContents = str_replace('%codigo%', $cdOperator, $replacedContents);
        $replacedContents = str_replace('%email%', $email, $replacedContents);
        $replacedContents = str_replace('%senha%', $password, $replacedContents);
        return $replacedContents;
    }
    
    /**
     * @param $style
     * @return string
     */
    private function getResetPasswordTemplatePath($style) 
    {
        $baseDir = __DIR__.'/../Util/Template/';
        $customFile = $baseDir . 'reset_password_' . $style . '.html';
        $defaultFile = $baseDir . 'email.html';
        return file_exists($customFile) ? $customFile : $defaultFile;
    }
    
    public function getProductLanguages($productId) 
    {
        $filter = array(
            "requestType" => "FilterData",
            "filter" => array( array(
                'name' => 'PRODUCT_ID', 'operator' => '=', 'value' => $productId
            ) )
        );
        
        return $this->utilities->callWebService(null, '/productLanguages', $filter, 'languages', $this->loginClientSecret, $this->loginUrl);
    }
    
    public function testLicenseConnection() 
    {
        $error = false;
        
        try {
            $this->utilities->callWebService(null, '/testConnection', array(), 'testConnection', $this->loginClientSecret, $this->loginUrl, false, true);
        } catch(\Exception $e) {
            $error = $e->getMessage();
        }
        
        return array (
            'ERROR' => $error,
            'URL'   => explode('/backend', $this->loginUrl)[0]
        );
    }
    
    public function getUserEmailByCdOperador($cdoperador) {
        $sql = "SELECT DSLOGINECO AS DSEMAILOPER FROM DADOSFORNECOMMERCE WHERE CDOPERFORN = :CDOPERADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            'CDOPERADOR' => $cdoperador
        ));
        return $stmt->fetch()['DSEMAILOPER'];
    }
    
    public function validateUser($cdoperador, $password) {
        $userEmail = $this->getUserEmailByCdOperador($cdoperador);
        $userData = $this->getUserData(array(
            'USER_EMAIL' => $userEmail,
            'PASSWORD'   => $password
        ));
        
        if(empty($userData)){
            return false;
        }
        
        self::getLibEnvironment()->setSessionExpired(false);
        return true;
    }
    
}